import functools
import imp
import logging
import os
import py_compile
import shutil
import sys
import tempfile
import warnings
import pkg_resources

log = logging.getLogger('chameleon.loader')

from .utils import string_type
from .utils import encode_string


def cache(func):
    def load(self, *args, **kwargs):
        template = self.registry.get(args)
        if template is None:
            self.registry[args] = template = func(self, *args, **kwargs)
        return template
    return load


def abspath_from_asset_spec(spec):
    pname, filename = spec.split(':', 1)
    return pkg_resources.resource_filename(pname, filename)

if os.name == "nt":
    def abspath_from_asset_spec(spec, f=abspath_from_asset_spec):
        if spec[1] == ":":
            return spec
        return f(spec)


class TemplateLoader(object):
    """Template loader class.

    To load templates using relative filenames, pass a sequence of
    paths (or a single path) as ``search_path``.

    To apply a default filename extension to inputs which do not have
    an extension already (i.e. no dot), provide this as
    ``default_extension`` (e.g. ``'.pt'``).

    Additional keyword-arguments will be passed on to the template
    constructor.
    """

    default_extension = None

    def __init__(self, search_path=None, default_extension=None, **kwargs):
        if search_path is None:
            search_path = []
        if isinstance(search_path, string_type):
            search_path = [search_path]
        if default_extension is not None:
            self.default_extension = ".%s" % default_extension.lstrip('.')
        self.search_path = search_path
        self.registry = {}
        self.kwargs = kwargs

    @cache
    def load(self, spec, cls=None):
        if cls is None:
            raise ValueError("Unbound template loader.")

        spec = spec.strip()

        if self.default_extension is not None and '.' not in spec:
            spec += self.default_extension

        if ':' in spec:
            spec = abspath_from_asset_spec(spec)

        if os.path.isabs(spec):
            return cls(spec, **self.kwargs)

        for path in self.search_path:
            path = os.path.join(path, spec)
            if os.path.exists(path):
                return cls(path, **self.kwargs)

        raise ValueError("Template not found: %s." % spec)

    def bind(self, cls):
        return functools.partial(self.load, cls=cls)


class MemoryLoader(object):
    def build(self, source, filename):
        code = compile(source, filename, 'exec')
        env = {}
        exec(code, env)
        return env

    def get(self, name):
        return None


class ModuleLoader(object):
    def __init__(self, path, remove=False):
        self.path = path
        self.remove = remove

    def __del__(self, shutil=shutil):
        if not self.remove:
            return
        try:
            shutil.rmtree(self.path)
        except:
            warnings.warn("Could not clean up temporary file path: %s" % (self.path,))

    def get(self, filename):
        path = os.path.join(self.path, filename)
        if os.path.exists(path):
            log.debug("loading module from cache: %s." % filename)
            base, ext = os.path.splitext(filename)
            return self._load(base, path)
        else:
            log.debug('cache miss: %s' % filename)

    def build(self, source, filename):
        imp.acquire_lock()
        try:
            d = self.get(filename)
            if d is not None:
                return d

            base, ext = os.path.splitext(filename)
            name = os.path.join(self.path, base + ".py")

            log.debug("writing source to disk (%d bytes)." % len(source))
            fd, fn = tempfile.mkstemp(prefix=base, suffix='.tmp', dir=self.path)
            temp = os.fdopen(fd, 'wb')
            encoded = source.encode('utf-8')
            header = encode_string("# -*- coding: utf-8 -*-" + "\n")

            try:
                try:
                    temp.write(header)
                    temp.write(encoded)
                finally:
                    temp.close()
            except:
                os.remove(fn)
                raise

            os.rename(fn, name)
            log.debug("compiling %s into byte-code..." % filename)
            py_compile.compile(name)

            return self._load(base, name)
        finally:
            imp.release_lock()

    def _load(self, base, filename):
        imp.acquire_lock()
        try:
            module = sys.modules.get(base)
            if module is None:
                f = open(filename, 'rb')
                try:
                    assert base not in sys.modules
                    module = imp.load_source(base, filename, f)
                finally:
                    f.close()
        finally:
            imp.release_lock()

        return module.__dict__



# -*- coding: utf-8 -*-
pass
from chameleon.utils import Placeholder as _Placeholder
import sys as _sys
pass
_static_38401872 = {}
_marker_default = _Placeholder()
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_35793880 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x249f750> name=None at 249f810> -> _value
    _value = _static_38401872
    econtext['attrs'] = _value

    # <div ... (1:0)
    # --------------------------------------------------------
    append(u'<div>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_40161008 = get('default', _marker)

    # <Marker name='default' at 2658ad0> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u"'Hello World!'" (2:24)> -> _cache_40208016
    try:
        _cache_40208016 = 'Hello World!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello World!'", 2, 24, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u"'Hello World!'" (2:24)> value=<Marker name='default' at 2658710> at 2658990> -> _condition
    _expression = _cache_40208016

    # <Marker name='default' at 2658710> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        pass
    else:
        _content = _cache_40208016
        if (_content is None):
            pass
        else:
            if (_content is False):
                _content = None
            else:
                _tt = type(_content)
                if ((_tt is int) or (_tt is float) or (_tt is long)):
                    _content = unicode(_content)
                else:
                    try:
                        if (_tt is str):
                            _content = decode(_content)
                        else:
                            if (_tt is not unicode):
                                try:
                                    _content = _content.__html__
                                except:
                                    _content = convert(_content)
                                else:
                                    raise RuntimeError
                    except RuntimeError:
                        _content = _content()
                    else:
                        if ((_content is not None) and (re_needs_escape(_content) is not None)):
                            if ('&' in _content):
                                if (';' in _content):
                                    _content = re_amp.sub('&amp;', _content)
                                else:
                                    _content = _content.replace('&', '&amp;')
                            if ('<' in _content):
                                _content = _content.replace('<', '&lt;')
                            if ('>' in _content):
                                _content = _content.replace('>', '&gt;')
                            if ('\x00' in _content):
                                _content = _content.replace('\x00', '&#34;')
        if (_content is not None):
            append(_content)
    if (_backup_default_40161008 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_40161008
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_35794744 = get('default', _marker)

    # <Marker name='default' at 2521610> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u"'Hello World!'" (3:28)> -> _cache_36787408
    try:
        _cache_36787408 = 'Hello World!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello World!'", 3, 28, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u"'Hello World!'" (3:28)> value=<Marker name='default' at 2521f90> at 2521cd0> -> _condition
    _expression = _cache_36787408

    # <Marker name='default' at 2521f90> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        pass
    else:
        _content = _cache_36787408
        if (_content is None):
            pass
        else:
            if (_content is False):
                _content = None
            else:
                _tt = type(_content)
                if ((_tt is int) or (_tt is float) or (_tt is long)):
                    _content = unicode(_content)
                else:
                    try:
                        if (_tt is str):
                            _content = decode(_content)
                        else:
                            if (_tt is not unicode):
                                try:
                                    _content = _content.__html__
                                except:
                                    _content = convert(_content)
                                else:
                                    raise RuntimeError
                    except RuntimeError:
                        _content = _content()
                    else:
                        if ((_content is not None) and (re_needs_escape(_content) is not None)):
                            if ('&' in _content):
                                if (';' in _content):
                                    _content = re_amp.sub('&amp;', _content)
                                else:
                                    _content = _content.replace('&', '&amp;')
                            if ('<' in _content):
                                _content = _content.replace('<', '&lt;')
                            if ('>' in _content):
                                _content = _content.replace('>', '&gt;')
                            if ('\x00' in _content):
                                _content = _content.replace('\x00', '&#34;')
        if (_content is not None):
            append(_content)
    if (_backup_default_35794744 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_35794744
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_35792944 = get('default', _marker)

    # <Marker name='default' at 25219d0> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u"'Hello World!'" (4:20)> -> _cache_40209232
    try:
        _cache_40209232 = 'Hello World!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello World!'", 4, 20, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u"'Hello World!'" (4:20)> value=<Marker name='default' at 2521810> at 2521910> -> _condition
    _expression = _cache_40209232

    # <Marker name='default' at 2521810> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        pass
    else:
        _content = _cache_40209232
        if (_content is None):
            pass
        else:
            if (_content is False):
                _content = None
            else:
                _tt = type(_content)
                if ((_tt is int) or (_tt is float) or (_tt is long)):
                    _content = unicode(_content)
                else:
                    try:
                        if (_tt is str):
                            _content = decode(_content)
                        else:
                            if (_tt is not unicode):
                                try:
                                    _content = _content.__html__
                                except:
                                    _content = convert(_content)
                                else:
                                    raise RuntimeError
                    except RuntimeError:
                        _content = _content()
                    else:
                        if ((_content is not None) and (re_needs_escape(_content) is not None)):
                            if ('&' in _content):
                                if (';' in _content):
                                    _content = re_amp.sub('&amp;', _content)
                                else:
                                    _content = _content.replace('&', '&amp;')
                            if ('<' in _content):
                                _content = _content.replace('<', '&lt;')
                            if ('>' in _content):
                                _content = _content.replace('>', '&gt;')
                            if ('\x00' in _content):
                                _content = _content.replace('\x00', '&#34;')
        if (_content is not None):
            append(_content)
    if (_backup_default_35792944 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_35792944
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_i_40210256 = get('i', _marker)

    # <Expression u'range(3)' (5:26)> -> _iterator
    try:
        _iterator = get('range', range)(3)
    except:
        rcontext.setdefault('__error__', []).append((u'range(3)', 5, 26, '<string>', _sys.exc_info()[1], ))
        raise

    (_iterator, __index_38934736, ) = getitem('repeat')(u'i', _iterator)
    econtext['i'] = None
    for _item in _iterator:
        econtext['i'] = _item
        _backup_default_40162008 = get('default', _marker)

        # <Marker name='default' at 2521f50> -> _value
        _value = _marker_default
        econtext['default'] = _value

        # <Expression u'i' (5:45)> -> _cache_38932816
        try:
            _cache_38932816 = getitem('i')
        except:
            rcontext.setdefault('__error__', []).append((u'i', 5, 45, '<string>', _sys.exc_info()[1], ))
            raise


        # <Identity expression=<Expression u'i' (5:45)> value=<Marker name='default' at 2521050> at 25213d0> -> _condition
        _expression = _cache_38932816

        # <Marker name='default' at 2521050> -> _value
        _value = _marker_default
        _condition = (_expression is _value)
        if _condition:
            pass
        else:
            _content = _cache_38932816
            if (_content is None):
                pass
            else:
                if (_content is False):
                    _content = None
                else:
                    _tt = type(_content)
                    if ((_tt is int) or (_tt is float) or (_tt is long)):
                        _content = unicode(_content)
                    else:
                        try:
                            if (_tt is str):
                                _content = decode(_content)
                            else:
                                if (_tt is not unicode):
                                    try:
                                        _content = _content.__html__
                                    except:
                                        _content = convert(_content)
                                    else:
                                        raise RuntimeError
                        except RuntimeError:
                            _content = _content()
                        else:
                            if ((_content is not None) and (re_needs_escape(_content) is not None)):
                                if ('&' in _content):
                                    if (';' in _content):
                                        _content = re_amp.sub('&amp;', _content)
                                    else:
                                        _content = _content.replace('&', '&amp;')
                                if ('<' in _content):
                                    _content = _content.replace('<', '&lt;')
                                if ('>' in _content):
                                    _content = _content.replace('>', '&gt;')
                                if ('\x00' in _content):
                                    _content = _content.replace('\x00', '&#34;')
            if (_content is not None):
                append(_content)
        if (_backup_default_40162008 is _marker):
            del econtext['default']
        else:
            econtext['default'] = _backup_default_40162008
        __index_38934736 -= 1
        if (__index_38934736 > 0):
            append('\n  ')
    if (_backup_i_40210256 is _marker):
        del econtext['i']
    else:
        econtext['i'] = _backup_i_40210256
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)

    # <Expression u'True' (6:22)> -> _condition
    try:
        _condition = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 6, 22, '<string>', _sys.exc_info()[1], ))
        raise

    if _condition:
        _content_139955154988272 = u'True'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_35793880 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_35793880
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-
pass
import sys as _sys
pass
_static_37141392 = {}
_static_38401104 = {}
_static_38400656 = {}
_static_37139024 = {}
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_38619040 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x249f290> name=None at 249fc10> -> _value
    _value = _static_38400656
    econtext['attrs'] = _value

    # <html ... (1:0)
    # --------------------------------------------------------
    append(u'<html>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)

    # <Expression u'True' (2:23)> -> _condition
    try:
        _condition = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 2, 23, '<string>', _sys.exc_info()[1], ))
        raise

    if _condition:
        _backup_attrs_39065432 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x249f450> name=None at 249f850> -> _value
        _value = _static_38401104
        econtext['attrs'] = _value

        # <body ... (2:2)
        # --------------------------------------------------------
        append(u'<body>')
        _content_139955154988272 = u'\n    '
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        _backup_selector_38402576 = get('selector', _marker)

        # <Expression u'False' (3:31)> -> _value
        try:
            _value = False
        except:
            rcontext.setdefault('__error__', []).append((u'False', 3, 31, '<string>', _sys.exc_info()[1], ))
            raise

        econtext['selector'] = _value

        # <Expression u'selector' (3:53)> -> _condition
        try:
            _condition = getitem('selector')
        except:
            rcontext.setdefault('__error__', []).append((u'selector', 3, 53, '<string>', _sys.exc_info()[1], ))
            raise

        if _condition:
            _backup_attrs_39065576 = get('attrs', _marker)

            # <Static value=<_ast.Dict object at 0x236b250> name=None at 236b450> -> _value
            _value = _static_37139024
            econtext['attrs'] = _value

            # <span ... (3:4)
            # --------------------------------------------------------
            append(u'<span>')
            _content_139955154988272 = u'bad'
            if (_content_139955154988272 is not None):
                append(_content_139955154988272)
            append(u'</span>')
            if (_backup_attrs_39065576 is _marker):
                del econtext['attrs']
            else:
                econtext['attrs'] = _backup_attrs_39065576
        if (_backup_selector_38402576 is _marker):
            del econtext['selector']
        else:
            econtext['selector'] = _backup_selector_38402576
        _content_139955154988272 = u'\n    '
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)

        # <Expression u'True' (4:25)> -> _condition
        try:
            _condition = True
        except:
            rcontext.setdefault('__error__', []).append((u'True', 4, 25, '<string>', _sys.exc_info()[1], ))
            raise

        if _condition:
            _backup_attrs_39064784 = get('attrs', _marker)

            # <Static value=<_ast.Dict object at 0x236bb90> name=None at 236bbd0> -> _value
            _value = _static_37141392
            econtext['attrs'] = _value

            # <span ... (4:4)
            # --------------------------------------------------------
            append(u'<span>')
            _content_139955154988272 = u'ok'
            if (_content_139955154988272 is not None):
                append(_content_139955154988272)
            append(u'</span>')
            if (_backup_attrs_39064784 is _marker):
                del econtext['attrs']
            else:
                econtext['attrs'] = _backup_attrs_39064784
        _content_139955154988272 = u'\n  '
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</body>')
        if (_backup_attrs_39065432 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_39065432
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</html>')
    if (_backup_attrs_38619040 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38619040
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


#!/usr/bin/python

"""
Benchmark for test the performance of Mako templates engine.
Includes:
    -two template inherences
    -HTML escaping, XML escaping, URL escaping, whitespace trimming
    -function defitions and calls
    -forloops
"""

__author__ = "virhilo@gmail.com (Lukasz Fidosz)"

# Python imports
import os
import sys
import optparse
import time

# Local imports
import util

def relative(*args):
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), *args)

sys.path.insert(0, relative('..', 'lib'))

# Mako imports
from mako.template import Template
from mako.lookup import TemplateLookup


LOREM_IPSUM = """Quisque lobortis hendrerit posuere. Curabitur
aliquet consequat sapien molestie pretium. Nunc adipiscing luc
tus mi, viverra porttitor lorem vulputate et. Ut at purus sem,
sed tincidunt ante. Vestibulum ante ipsum primis in faucibus 
orci luctus et ultrices posuere cubilia Curae; Praesent pulvinar
sodales justo at congue. Praesent aliquet facilisis nisl a
molestie. Sed tempus nisl ut augue eleifend tincidunt. Sed a
lacinia nulla. Cras tortor est, mollis et consequat at,
vulputate et orci. Nulla sollicitudin"""

BASE_TEMPLATE = """
<%def name="render_table(table)">
    <table>
    % for row in table:
        <tr>
        % for col in row:
            <td>${col|h}</td>
        % endfor
        </tr>
    % endfor
    </table>
</%def>
<%def name="img(src, alt)">
    <img src="${src|u}" alt="${alt}" />
</%def>
<html>
    <head><title>${title|h,trim}</title></head>
    <body>
        ${next.body()}
    </body>
<html>
"""

PAGE_TEMPLATE = """
<%inherit file="base.mako"/>
<table>
    % for row in table:
        <tr>
            % for col in row:
                <td>${col}</td>
            % endfor
        </tr>
    % endfor
</table>
% for nr in xrange(img_count):
    ${parent.img('/foo/bar/baz.png', 'no image :o')}
% endfor
${next.body()}
% for nr in paragraphs:
    <p>${lorem|x}</p>
% endfor
${parent.render_table(table)}
"""

CONTENT_TEMPLATE = """
<%inherit file="page.mako"/>
<%def name="fun1()">
    <span>fun1</span>
</%def>
<%def name="fun2()">
    <span>fun2</span>
</%def>
<%def name="fun3()">
    <span>foo3</span>
</%def>
<%def name="fun4()">
    <span>foo4</span>
</%def>
<%def name="fun5()">
    <span>foo5</span>
</%def>
<%def name="fun6()">
    <span>foo6</span>
</%def>
<p>Lorem ipsum dolor sit amet, consectetur adipiscing elit.
Nam laoreet justo in velit faucibus lobortis. Sed dictum sagittis
volutpat. Sed adipiscing vestibulum consequat. Nullam laoreet, ante
nec pretium varius, libero arcu porttitor orci, id cursus odio nibh
nec leo. Vestibulum dapibus pellentesque purus, sed bibendum tortor
laoreet id. Praesent quis sodales ipsum. Fusce ut ligula sed diam
pretium sagittis vel at ipsum. Nulla sagittis sem quam, et volutpat
velit. Fusce dapibus ligula quis lectus ultricies tempor. Pellente</p>
${fun1()}
${fun2()}
${fun3()}
${fun4()}
${fun5()}
${fun6()}
"""


def test_mako(count):

    lookup = TemplateLookup()
    lookup.put_string('base.mako', BASE_TEMPLATE)
    lookup.put_string('page.mako', PAGE_TEMPLATE)

    template = Template(CONTENT_TEMPLATE, lookup=lookup)
    
    table = [xrange(150) for i in xrange(150)]
    paragraphs = xrange(50)
    title = 'Hello world!'

    times = []
    for i in range(count):
        t0 = time.time()
        data = template.render(table=table, paragraphs=paragraphs,
                               lorem=LOREM_IPSUM, title=title,
                               img_count=50)
        t1 = time.time()
        times.append(t1-t0)
    return times

if __name__ == "__main__":
    parser = optparse.OptionParser(
        usage="%prog [options]",
        description=("Test the performance of Mako templates."))
    util.add_standard_options_to(parser)
    (options, args) = parser.parse_args()

    util.run_benchmark(options, options.num_runs, test_mako)



# -*- coding: utf-8 -*-
pass
from chameleon.utils import Placeholder as _Placeholder
import sys as _sys
from chameleon.utils import DebuggingOutputStream as _DebuggingOutputStream
pass
_static_38577168 = {u'xmlns': u'http://www.w3.org/1999/xhtml', }
_marker_default = _Placeholder()
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_38450168 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24ca410> name=None at 24ca490> -> _value
    _value = _static_38577168
    econtext['attrs'] = _value

    # <div ... (1:0)
    # --------------------------------------------------------
    append(u'<div')
    _attr_xmlns = u'http://www.w3.org/1999/xhtml'
    if (_attr_xmlns is not None):
        append((u' xmlns="%s"' % _attr_xmlns))
    append(u'>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _stream_35214320_year = ''
    _stream_35214320_monthname = ''
    _stream_38579664 = _DebuggingOutputStream()
    _append_38579664 = _stream_38579664.append
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        _append_38579664(_content_139955154988272)
    _stream_35214320_monthname = _DebuggingOutputStream()
    _append_35214320_monthname = _stream_35214320_monthname.append
    _backup_default_38618392 = get('default', _marker)

    # <Marker name='default' at 24ae690> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u"'october'" (6:25)> -> _cache_38463888
    try:
        _cache_38463888 = 'october'
    except:
        rcontext.setdefault('__error__', []).append((u"'october'", 6, 25, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u"'october'" (6:25)> value=<Marker name='default' at 24ae150> at 24ae890> -> _condition
    _expression = _cache_38463888

    # <Marker name='default' at 24ae150> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        _content_139955154988272 = u'monthname'
        if (_content_139955154988272 is not None):
            _append_35214320_monthname(_content_139955154988272)
    else:
        _content = _cache_38463888
        _content = translate(_content, default=None, domain=_i18n_domain)
        if (_content is not None):
            _append_35214320_monthname(_content)
    if (_backup_default_38618392 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_38618392
    _append_38579664(u'${monthname}')
    _stream_35214320_monthname = ''.join(_stream_35214320_monthname)
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        _append_38579664(_content_139955154988272)
    _stream_35214320_year = _DebuggingOutputStream()
    _append_35214320_year = _stream_35214320_year.append
    _backup_default_36748968 = get('default', _marker)

    # <Marker name='default' at 24a5dd0> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u'1982' (10:25)> -> _cache_38427600
    try:
        _cache_38427600 = 1982
    except:
        rcontext.setdefault('__error__', []).append((u'1982', 10, 25, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u'1982' (10:25)> value=<Marker name='default' at 24a5410> at 24a5f90> -> _condition
    _expression = _cache_38427600

    # <Marker name='default' at 24a5410> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        _content_139955154988272 = u'year'
        if (_content_139955154988272 is not None):
            _append_35214320_year(_content_139955154988272)
    else:
        _content = _cache_38427600
        _content = translate(_content, default=None, domain=_i18n_domain)
        if (_content is not None):
            _append_35214320_year(_content)
    if (_backup_default_36748968 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_36748968
    _append_38579664(u'${year}')
    _stream_35214320_year = ''.join(_stream_35214320_year)
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        _append_38579664(_content_139955154988272)
    _msgid_38579664 = re_whitespace(''.join(_stream_38579664)).strip()
    append(translate(_msgid_38579664, mapping={u'monthname': _stream_35214320_monthname, u'year': _stream_35214320_year, }, default=_msgid_38579664, domain=_i18n_domain))
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_38450168 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38450168
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-
pass
import sys as _sys
pass
_static_37142480 = {}
_static_38891664 = {}
_static_37142352 = {}
_static_38894800 = {}
_static_38893584 = {}
_static_36788624 = {}
_static_37141584 = {}
_static_36789456 = {}
_static_38400400 = {}
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_39791864 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2315cd0> name=None at 2315090> -> _value
    _value = _static_36789456
    econtext['attrs'] = _value

    # <html ... (1:0)
    # --------------------------------------------------------
    append(u'<html>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_39094824 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2315990> name=None at 2315f90> -> _value
    _value = _static_36788624
    econtext['attrs'] = _value

    # <body ... (2:2)
    # --------------------------------------------------------
    append(u'<body>')
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)

    # <Expression u'True' (3:21)> -> _cache_37139408
    try:
        _cache_37139408 = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 3, 21, '<string>', _sys.exc_info()[1], ))
        raise

    _backup_attrs_38450528 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x236bc50> name=None at 236b150> -> _value
    _value = _static_37141584
    econtext['attrs'] = _value

    # <div ... (3:4)
    # --------------------------------------------------------
    append(u'<div>')
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_35792944 = get('default', _marker)
    _value = _cache_37139408
    econtext['default'] = _value

    # <Equality expression=<Expression u'True' (3:21)> value=<Expression u'False' (4:22)> at 236bcd0> -> _condition
    _expression = _cache_37139408

    # <Expression u'False' (4:22)> -> _value
    try:
        _value = False
    except:
        rcontext.setdefault('__error__', []).append((u'False', 4, 22, '<string>', _sys.exc_info()[1], ))
        raise

    _condition = (_expression == _value)
    if _condition:
        _backup_attrs_35791864 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x236bf50> name=None at 236b210> -> _value
        _value = _static_37142352
        econtext['attrs'] = _value

        # <span ... (4:6)
        # --------------------------------------------------------
        append(u'<span>')
        _content_139955154988272 = u'bad'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</span>')
        if (_backup_attrs_35791864 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_35791864
    if (_backup_default_35792944 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_35792944
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_35794240 = get('default', _marker)
    _value = _cache_37139408
    econtext['default'] = _value

    # <Equality expression=<Expression u'True' (3:21)> value=<Expression u'True' (5:22)> at 249f050> -> _condition
    _expression = _cache_37139408

    # <Expression u'True' (5:22)> -> _value
    try:
        _value = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 5, 22, '<string>', _sys.exc_info()[1], ))
        raise

    _condition = (_expression == _value)
    if _condition:
        _backup_attrs_35794024 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x236bfd0> name=None at 236bb10> -> _value
        _value = _static_37142480
        econtext['attrs'] = _value

        # <span ... (5:6)
        # --------------------------------------------------------
        append(u'<span>')
        _content_139955154988272 = u'ok'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</span>')
        if (_backup_attrs_35794024 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_35794024
    if (_backup_default_35794240 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_35794240
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_35792224 = get('default', _marker)
    _value = _cache_37139408
    econtext['default'] = _value

    # <Equality expression=<Expression u'True' (3:21)> value=<Expression u'not not True' (6:22)> at 249fc50> -> _condition
    _expression = _cache_37139408

    # <Expression u'not not True' (6:22)> -> _value
    try:
        _value = not not True
    except:
        rcontext.setdefault('__error__', []).append((u'not not True', 6, 22, '<string>', _sys.exc_info()[1], ))
        raise

    _condition = (_expression == _value)
    if _condition:
        _backup_attrs_35794312 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x249f190> name=None at 249f950> -> _value
        _value = _static_38400400
        econtext['attrs'] = _value

        # <span ... (6:6)
        # --------------------------------------------------------
        append(u'<span>')
        _content_139955154988272 = u'ok'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</span>')
        if (_backup_attrs_35794312 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_35794312
    if (_backup_default_35792224 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_35792224
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_38450528 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38450528
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)

    # <Expression u'True' (8:21)> -> _cache_38892304
    try:
        _cache_38892304 = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 8, 21, '<string>', _sys.exc_info()[1], ))
        raise

    _backup_attrs_39792936 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2517090> name=None at 2517150> -> _value
    _value = _static_38891664
    econtext['attrs'] = _value

    # <div ... (8:4)
    # --------------------------------------------------------
    append(u'<div>')
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_39865448 = get('default', _marker)
    _value = _cache_38892304
    econtext['default'] = _value

    # <Equality expression=<Expression u'True' (8:21)> value=<Expression u'False' (9:22)> at 2517590> -> _condition
    _expression = _cache_38892304

    # <Expression u'False' (9:22)> -> _value
    try:
        _value = False
    except:
        rcontext.setdefault('__error__', []).append((u'False', 9, 22, '<string>', _sys.exc_info()[1], ))
        raise

    _condition = (_expression == _value)
    if _condition:
        _backup_attrs_39863216 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x2517810> name=None at 2517690> -> _value
        _value = _static_38893584
        econtext['attrs'] = _value

        # <span ... (9:6)
        # --------------------------------------------------------
        append(u'<span>')
        _content_139955154988272 = u'bad'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</span>')
        if (_backup_attrs_39863216 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_39863216
    if (_backup_default_39865448 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_39865448
    _content_139955154988272 = u'\n      '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_default_39804224 = get('default', _marker)
    _value = _cache_38892304
    econtext['default'] = _value

    # <Equality expression=<Expression u'True' (8:21)> value=<Expression u'default' (10:22)> at 2517a50> -> _condition
    _expression = _cache_38892304

    # <Expression u'default' (10:22)> -> _value
    try:
        _value = getitem('default')
    except:
        rcontext.setdefault('__error__', []).append((u'default', 10, 22, '<string>', _sys.exc_info()[1], ))
        raise

    _condition = (_expression == _value)
    if _condition:
        _backup_attrs_39865808 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x2517cd0> name=None at 2517b90> -> _value
        _value = _static_38894800
        econtext['attrs'] = _value

        # <span ... (10:6)
        # --------------------------------------------------------
        append(u'<span>')
        _content_139955154988272 = u'ok'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</span>')
        if (_backup_attrs_39865808 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_39865808
    if (_backup_default_39804224 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_39804224
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_39792936 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39792936
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</body>')
    if (_backup_attrs_39094824 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39094824
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</html>')
    if (_backup_attrs_39791864 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39791864
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-
pass
import sys as _sys
pass
_static_39101904 = {u'checked': u'True', }
_static_39101136 = {u'checked': u'False', }
_static_38932624 = {u'selected': u'None', }
_static_38934992 = {u'selected': u'True', }
_static_38934864 = {u'xmlns': u'http://www.w3.org/1999/xhtml', }
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_39545024 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2521950> name=None at 2521e90> -> _value
    _value = _static_38934864
    econtext['attrs'] = _value

    # <div ... (1:0)
    # --------------------------------------------------------
    append(u'<div')
    _attr_xmlns = u'http://www.w3.org/1999/xhtml'
    if (_attr_xmlns is not None):
        append((u' xmlns="%s"' % _attr_xmlns))
    append(u'>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_39545672 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x25219d0> name=None at 2521610> -> _value
    _value = _static_38934992
    econtext['attrs'] = _value

    # <option ... (3:2)
    # --------------------------------------------------------
    append(u'<option')
    _backup_default_39545096 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Expression u'True' (3:35)> -> _attr_selected
    try:
        _attr_selected = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 3, 35, '<string>', _sys.exc_info()[1], ))
        raise

    if (_attr_selected is None):
        pass
    else:
        if (_attr_selected is False):
            _attr_selected = None
        else:
            _tt = type(_attr_selected)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_selected = unicode(_attr_selected)
            else:
                try:
                    if (_tt is str):
                        _attr_selected = decode(_attr_selected)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_selected = _attr_selected.__html__
                            except:
                                _attr_selected = convert(_attr_selected)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_selected = _attr_selected()
                else:
                    if ((_attr_selected is not None) and (re_needs_escape(_attr_selected) is not None)):
                        if ('&' in _attr_selected):
                            if (';' in _attr_selected):
                                _attr_selected = re_amp.sub('&amp;', _attr_selected)
                            else:
                                _attr_selected = _attr_selected.replace('&', '&amp;')
                        if ('<' in _attr_selected):
                            _attr_selected = _attr_selected.replace('<', '&lt;')
                        if ('>' in _attr_selected):
                            _attr_selected = _attr_selected.replace('>', '&gt;')
                        if ('"' in _attr_selected):
                            _attr_selected = _attr_selected.replace('"', '&#34;')
    if (_attr_selected is not None):
        append((u' selected="%s"' % _attr_selected))
    if (_backup_default_39545096 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_39545096
    append(u'>')
    append(u'</option>')
    if (_backup_attrs_39545672 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39545672
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_39594032 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2521090> name=None at 25215d0> -> _value
    _value = _static_38932624
    econtext['attrs'] = _value

    # <option ... (4:2)
    # --------------------------------------------------------
    append(u'<option')
    _backup_default_39594608 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Expression u'None' (4:35)> -> _attr_selected
    try:
        _attr_selected = None
    except:
        rcontext.setdefault('__error__', []).append((u'None', 4, 35, '<string>', _sys.exc_info()[1], ))
        raise

    if (_attr_selected is None):
        pass
    else:
        if (_attr_selected is False):
            _attr_selected = None
        else:
            _tt = type(_attr_selected)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_selected = unicode(_attr_selected)
            else:
                try:
                    if (_tt is str):
                        _attr_selected = decode(_attr_selected)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_selected = _attr_selected.__html__
                            except:
                                _attr_selected = convert(_attr_selected)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_selected = _attr_selected()
                else:
                    if ((_attr_selected is not None) and (re_needs_escape(_attr_selected) is not None)):
                        if ('&' in _attr_selected):
                            if (';' in _attr_selected):
                                _attr_selected = re_amp.sub('&amp;', _attr_selected)
                            else:
                                _attr_selected = _attr_selected.replace('&', '&amp;')
                        if ('<' in _attr_selected):
                            _attr_selected = _attr_selected.replace('<', '&lt;')
                        if ('>' in _attr_selected):
                            _attr_selected = _attr_selected.replace('>', '&gt;')
                        if ('"' in _attr_selected):
                            _attr_selected = _attr_selected.replace('"', '&#34;')
    if (_attr_selected is not None):
        append((u' selected="%s"' % _attr_selected))
    if (_backup_default_39594608 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_39594608
    append(u'>')
    append(u'</option>')
    if (_backup_attrs_39594032 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39594032
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_39595400 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x254a5d0> name=None at 254a750> -> _value
    _value = _static_39101904
    econtext['attrs'] = _value

    # <input ... (5:2)
    # --------------------------------------------------------
    append(u'<input')
    _backup_default_40174800 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Expression u'True' (5:33)> -> _attr_checked
    try:
        _attr_checked = True
    except:
        rcontext.setdefault('__error__', []).append((u'True', 5, 33, '<string>', _sys.exc_info()[1], ))
        raise

    if (_attr_checked is None):
        pass
    else:
        if (_attr_checked is False):
            _attr_checked = None
        else:
            _tt = type(_attr_checked)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_checked = unicode(_attr_checked)
            else:
                try:
                    if (_tt is str):
                        _attr_checked = decode(_attr_checked)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_checked = _attr_checked.__html__
                            except:
                                _attr_checked = convert(_attr_checked)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_checked = _attr_checked()
                else:
                    if ((_attr_checked is not None) and (re_needs_escape(_attr_checked) is not None)):
                        if ('&' in _attr_checked):
                            if (';' in _attr_checked):
                                _attr_checked = re_amp.sub('&amp;', _attr_checked)
                            else:
                                _attr_checked = _attr_checked.replace('&', '&amp;')
                        if ('<' in _attr_checked):
                            _attr_checked = _attr_checked.replace('<', '&lt;')
                        if ('>' in _attr_checked):
                            _attr_checked = _attr_checked.replace('>', '&gt;')
                        if ('"' in _attr_checked):
                            _attr_checked = _attr_checked.replace('"', '&#34;')
    if (_attr_checked is not None):
        append((u' checked="%s"' % _attr_checked))
    if (_backup_default_40174800 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_40174800
    append(u' />')
    if (_backup_attrs_39595400 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39595400
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_39593744 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x254a2d0> name=None at 254a210> -> _value
    _value = _static_39101136
    econtext['attrs'] = _value

    # <input ... (6:2)
    # --------------------------------------------------------
    append(u'<input')
    _backup_default_39066584 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Expression u'False' (6:33)> -> _attr_checked
    try:
        _attr_checked = False
    except:
        rcontext.setdefault('__error__', []).append((u'False', 6, 33, '<string>', _sys.exc_info()[1], ))
        raise

    if (_attr_checked is None):
        pass
    else:
        if (_attr_checked is False):
            _attr_checked = None
        else:
            _tt = type(_attr_checked)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_checked = unicode(_attr_checked)
            else:
                try:
                    if (_tt is str):
                        _attr_checked = decode(_attr_checked)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_checked = _attr_checked.__html__
                            except:
                                _attr_checked = convert(_attr_checked)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_checked = _attr_checked()
                else:
                    if ((_attr_checked is not None) and (re_needs_escape(_attr_checked) is not None)):
                        if ('&' in _attr_checked):
                            if (';' in _attr_checked):
                                _attr_checked = re_amp.sub('&amp;', _attr_checked)
                            else:
                                _attr_checked = _attr_checked.replace('&', '&amp;')
                        if ('<' in _attr_checked):
                            _attr_checked = _attr_checked.replace('<', '&lt;')
                        if ('>' in _attr_checked):
                            _attr_checked = _attr_checked.replace('>', '&gt;')
                        if ('"' in _attr_checked):
                            _attr_checked = _attr_checked.replace('"', '&#34;')
    if (_attr_checked is not None):
        append((u' checked="%s"' % _attr_checked))
    if (_backup_default_39066584 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_39066584
    append(u' />')
    if (_backup_attrs_39593744 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39593744
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_39545024 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39545024
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-
pass
from chameleon.utils import Placeholder as _Placeholder
import sys as _sys
pass
_static_35707984 = {}
_static_35803088 = {}
_static_36671184 = {}
_static_35779600 = {}
_static_36670416 = {u'class': u'default', }
_static_36668432 = {u'class': u'default', }
_static_35802384 = {}
_marker_default = _Placeholder()
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_38643400 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2224d10> name=None at 2224a10> -> _value
    _value = _static_35802384
    econtext['attrs'] = _value

    # <html ... (1:0)
    # --------------------------------------------------------
    append(u'<html>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38494936 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2224fd0> name=None at 2224290> -> _value
    _value = _static_35803088
    econtext['attrs'] = _value

    # <body ... (2:2)
    # --------------------------------------------------------
    append(u'<body>')
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38642320 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x22f8410> name=None at 2224dd0> -> _value
    _value = _static_36668432
    econtext['attrs'] = _value

    # <img ... (3:4)
    # --------------------------------------------------------
    append(u'<img')
    _backup_default_38494504 = get('default', _marker)
    _value = u'default'
    econtext['default'] = _value

    # <Expression u'default' (3:47)> -> _attr_class
    try:
        _attr_class = getitem('default')
    except:
        rcontext.setdefault('__error__', []).append((u'default', 3, 47, '<string>', _sys.exc_info()[1], ))
        raise

    if (_attr_class is None):
        pass
    else:
        if (_attr_class is False):
            _attr_class = None
        else:
            _tt = type(_attr_class)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_class = unicode(_attr_class)
            else:
                try:
                    if (_tt is str):
                        _attr_class = decode(_attr_class)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_class = _attr_class.__html__
                            except:
                                _attr_class = convert(_attr_class)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_class = _attr_class()
                else:
                    if ((_attr_class is not None) and (re_needs_escape(_attr_class) is not None)):
                        if ('&' in _attr_class):
                            if (';' in _attr_class):
                                _attr_class = re_amp.sub('&amp;', _attr_class)
                            else:
                                _attr_class = _attr_class.replace('&', '&amp;')
                        if ('<' in _attr_class):
                            _attr_class = _attr_class.replace('<', '&lt;')
                        if ('>' in _attr_class):
                            _attr_class = _attr_class.replace('>', '&gt;')
                        if (u'"' in _attr_class):
                            _attr_class = _attr_class.replace(u'"', '&#34;')
    if (_attr_class is not None):
        append((u' class="%s"' % _attr_class))
    if (_backup_default_38494504 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_38494504
    append(u' />')
    if (_backup_attrs_38642320 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38642320
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38495800 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x22f8bd0> name=None at 22f8810> -> _value
    _value = _static_36670416
    econtext['attrs'] = _value

    # <img ... (4:4)
    # --------------------------------------------------------
    append(u'<img')
    _backup_default_38495728 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Expression u'default' (4:31)> -> _attr_class
    try:
        _attr_class = getitem('default')
    except:
        rcontext.setdefault('__error__', []).append((u'default', 4, 31, '<string>', _sys.exc_info()[1], ))
        raise

    if (_attr_class is None):
        pass
    else:
        if (_attr_class is False):
            _attr_class = None
        else:
            _tt = type(_attr_class)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_class = unicode(_attr_class)
            else:
                try:
                    if (_tt is str):
                        _attr_class = decode(_attr_class)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_class = _attr_class.__html__
                            except:
                                _attr_class = convert(_attr_class)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_class = _attr_class()
                else:
                    if ((_attr_class is not None) and (re_needs_escape(_attr_class) is not None)):
                        if ('&' in _attr_class):
                            if (';' in _attr_class):
                                _attr_class = re_amp.sub('&amp;', _attr_class)
                            else:
                                _attr_class = _attr_class.replace('&', '&amp;')
                        if ('<' in _attr_class):
                            _attr_class = _attr_class.replace('<', '&lt;')
                        if ('>' in _attr_class):
                            _attr_class = _attr_class.replace('>', '&gt;')
                        if ('"' in _attr_class):
                            _attr_class = _attr_class.replace('"', '&#34;')
    if (_attr_class is not None):
        append((u' class="%s"' % _attr_class))
    if (_backup_default_38495728 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_38495728
    append(u' />')
    if (_backup_attrs_38495800 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38495800
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38571832 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x22f8ed0> name=None at 22f8f90> -> _value
    _value = _static_36671184
    econtext['attrs'] = _value

    # <span ... (5:4)
    # --------------------------------------------------------
    append(u'<span>')
    _backup_default_37019016 = get('default', _marker)

    # <Marker name='default' at 22f8890> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u'default' (5:23)> -> _cache_36670032
    try:
        _cache_36670032 = getitem('default')
    except:
        rcontext.setdefault('__error__', []).append((u'default', 5, 23, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u'default' (5:23)> value=<Marker name='default' at 22f8990> at 22f89d0> -> _condition
    _expression = _cache_36670032

    # <Marker name='default' at 22f8990> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        _content_139955154988272 = u'Default'
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
    else:
        _content = _cache_36670032
        if (_content is None):
            pass
        else:
            if (_content is False):
                _content = None
            else:
                _tt = type(_content)
                if ((_tt is int) or (_tt is float) or (_tt is long)):
                    _content = unicode(_content)
                else:
                    try:
                        if (_tt is str):
                            _content = decode(_content)
                        else:
                            if (_tt is not unicode):
                                try:
                                    _content = _content.__html__
                                except:
                                    _content = convert(_content)
                                else:
                                    raise RuntimeError
                    except RuntimeError:
                        _content = _content()
                    else:
                        if ((_content is not None) and (re_needs_escape(_content) is not None)):
                            if ('&' in _content):
                                if (';' in _content):
                                    _content = re_amp.sub('&amp;', _content)
                                else:
                                    _content = _content.replace('&', '&amp;')
                            if ('<' in _content):
                                _content = _content.replace('<', '&lt;')
                            if ('>' in _content):
                                _content = _content.replace('>', '&gt;')
                            if ('\x00' in _content):
                                _content = _content.replace('\x00', '&#34;')
        if (_content is not None):
            append(_content)
    if (_backup_default_37019016 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_37019016
    append(u'</span>')
    if (_backup_attrs_38571832 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38571832
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38604240 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x220dc50> name=None at 22f8e90> -> _value
    _value = _static_35707984
    econtext['attrs'] = _value

    # <span ... (6:4)
    # --------------------------------------------------------
    append(u'<span>')
    _backup_default_38601360 = get('default', _marker)

    # <Marker name='default' at 22f8b50> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u'default' (6:23)> -> _cache_36670672
    try:
        _cache_36670672 = getitem('default')
    except:
        rcontext.setdefault('__error__', []).append((u'default', 6, 23, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u'default' (6:23)> value=<Marker name='default' at 22f88d0> at 22f8590> -> _condition
    _expression = _cache_36670672

    # <Marker name='default' at 22f88d0> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        _content_139955154988272 = u'\n      '
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        _backup_attrs_38632624 = get('attrs', _marker)

        # <Static value=<_ast.Dict object at 0x221f410> name=None at 221f490> -> _value
        _value = _static_35779600
        econtext['attrs'] = _value

        # <em ... (7:6)
        # --------------------------------------------------------
        append(u'<em>')

        # <Expression u"'Computed default'" (7:12)> -> _content_139955154988272
        try:
            _content_139955154988272 = 'Computed default'
        except:
            rcontext.setdefault('__error__', []).append((u"'Computed default'", 7, 12, '<string>', _sys.exc_info()[1], ))
            raise

        if (_content_139955154988272 is None):
            pass
        else:
            if (_content_139955154988272 is False):
                _content_139955154988272 = None
            else:
                _tt = type(_content_139955154988272)
                if ((_tt is int) or (_tt is float) or (_tt is long)):
                    _content_139955154988272 = unicode(_content_139955154988272)
                else:
                    try:
                        if (_tt is str):
                            _content_139955154988272 = decode(_content_139955154988272)
                        else:
                            if (_tt is not unicode):
                                try:
                                    _content_139955154988272 = _content_139955154988272.__html__
                                except:
                                    _content_139955154988272 = convert(_content_139955154988272)
                                else:
                                    raise RuntimeError
                    except RuntimeError:
                        _content_139955154988272 = _content_139955154988272()
                    else:
                        if ((_content_139955154988272 is not None) and (re_needs_escape(_content_139955154988272) is not None)):
                            if ('&' in _content_139955154988272):
                                if (';' in _content_139955154988272):
                                    _content_139955154988272 = re_amp.sub('&amp;', _content_139955154988272)
                                else:
                                    _content_139955154988272 = _content_139955154988272.replace('&', '&amp;')
                            if ('<' in _content_139955154988272):
                                _content_139955154988272 = _content_139955154988272.replace('<', '&lt;')
                            if ('>' in _content_139955154988272):
                                _content_139955154988272 = _content_139955154988272.replace('>', '&gt;')
                            if ('\x00' in _content_139955154988272):
                                _content_139955154988272 = _content_139955154988272.replace('\x00', '&#34;')
        _content_139955154988272 = _content_139955154988272
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
        append(u'</em>')
        if (_backup_attrs_38632624 is _marker):
            del econtext['attrs']
        else:
            econtext['attrs'] = _backup_attrs_38632624
        _content_139955154988272 = u'\n    '
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
    else:
        _content = _cache_36670672
        if (_content is None):
            pass
        else:
            if (_content is False):
                _content = None
            else:
                _tt = type(_content)
                if ((_tt is int) or (_tt is float) or (_tt is long)):
                    _content = unicode(_content)
                else:
                    try:
                        if (_tt is str):
                            _content = decode(_content)
                        else:
                            if (_tt is not unicode):
                                try:
                                    _content = _content.__html__
                                except:
                                    _content = convert(_content)
                                else:
                                    raise RuntimeError
                    except RuntimeError:
                        _content = _content()
                    else:
                        if ((_content is not None) and (re_needs_escape(_content) is not None)):
                            if ('&' in _content):
                                if (';' in _content):
                                    _content = re_amp.sub('&amp;', _content)
                                else:
                                    _content = _content.replace('&', '&amp;')
                            if ('<' in _content):
                                _content = _content.replace('<', '&lt;')
                            if ('>' in _content):
                                _content = _content.replace('>', '&gt;')
                            if ('\x00' in _content):
                                _content = _content.replace('\x00', '&#34;')
        if (_content is not None):
            append(_content)
    if (_backup_default_38601360 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_38601360
    append(u'</span>')
    if (_backup_attrs_38604240 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38604240
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</body>')
    if (_backup_attrs_38494936 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38494936
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</html>')
    if (_backup_attrs_38643400 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38643400
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-

from __future__ import with_statement

import re
import os
import sys
import shutil
import tempfile

from functools import wraps
from functools import partial

try:
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase


from chameleon.utils import byte_string


class Message(object):
    def __str__(self):
        return "message"


class ImportTestCase(TestCase):
    def test_pagetemplates(self):
        from chameleon import PageTemplate
        from chameleon import PageTemplateFile
        from chameleon import PageTemplateLoader

    def test_pagetexttemplates(self):
        from chameleon import PageTextTemplate
        from chameleon import PageTextTemplateFile


class TemplateFileTestCase(TestCase):
    @property
    def _class(self):
        from chameleon.template import BaseTemplateFile

        class TestTemplateFile(BaseTemplateFile):
            cook_count = 0

            def cook(self, body):
                self.cook_count += 1
                self._cooked = True

        return TestTemplateFile

    def setUp(self):
        self.tempdir = tempfile.mkdtemp(prefix='chameleon-tests')

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def _get_temporary_file(self):
        filename = os.path.join(self.tempdir, 'template.py')
        assert not os.path.exists(filename)
        f = open(filename, 'w')
        f.flush()
        f.close()
        return filename

    def test_cook_check(self):
        fn = self._get_temporary_file()
        template = self._class(fn)
        template.cook_check()
        self.assertEqual(template.cook_count, 1)

    def test_auto_reload(self):
        fn = self._get_temporary_file()

        # set time in past
        os.utime(fn, (0, 0))

        template = self._class(fn, auto_reload=True)
        template.cook_check()

        # a second cook check makes no difference
        template.cook_check()
        self.assertEqual(template.cook_count, 1)

        # set current time on file
        os.utime(fn, None)

        # file is reloaded
        template.cook_check()
        self.assertEqual(template.cook_count, 2)

    def test_relative_is_expanded_to_cwd(self):
        template = self._class("___does_not_exist___")
        try:
            template.cook_check()
        except IOError:
            exc = sys.exc_info()[1]
            self.assertEqual(
                os.getcwd(),
                os.path.dirname(exc.filename)
                )
        else:
            self.fail("Expected OSError.")


class RenderTestCase(TestCase):
    root = os.path.dirname(__file__)

    def find_files(self, ext):
        inputs = os.path.join(self.root, "inputs")
        outputs = os.path.join(self.root, "outputs")
        for filename in sorted(os.listdir(inputs)):
            name, extension = os.path.splitext(filename)
            if extension != ext:
                continue
            path = os.path.join(inputs, filename)

            # if there's no output file, treat document as static and
            # expect intput equal to output
            import glob
            globbed = tuple(glob.iglob(os.path.join(
                outputs, "%s*%s" % (name.split('-', 1)[0], ext))))

            if not globbed:
                self.fail("Missing output for: %s." % name)

            for output in globbed:
                name, ext = os.path.splitext(output)
                basename = os.path.basename(name)
                if '-' in basename:
                    language = basename.split('-')[1]
                else:
                    language = None

                yield path, output, language


class ZopePageTemplatesTest(RenderTestCase):
    @property
    def from_string(body):
        from ..zpt.template import PageTemplate
        return partial(PageTemplate, keep_source=True)

    @property
    def from_file(body):
        from ..zpt.template import PageTemplateFile
        return partial(PageTemplateFile, keep_source=True)

    def template(body):
        def decorator(func):
            @wraps(func)
            def wrapper(self):
                template = self.from_string(body)
                return func(self, template)

            return wrapper
        return decorator

    def error(body):
        def decorator(func):
            @wraps(func)
            def wrapper(self):
                from chameleon.exc import TemplateError
                try:
                    template = self.from_string(body)
                except TemplateError:
                    exc = sys.exc_info()[1]
                    return func(self, body, exc)
                else:
                    self.fail("Expected exception.")

            return wrapper
        return decorator

    def test_syntax_error_in_strict_mode(self):
        from chameleon.exc import ExpressionError

        self.assertRaises(
            ExpressionError,
            self.from_string,
            """<tal:block replace='bad /// ' />""",
            strict=True
            )

    def test_syntax_error_in_non_strict_mode(self):
        from chameleon.exc import ExpressionError

        body = """<tal:block replace='bad /// ' />"""
        template = self.from_string(body, strict=False)

        try:
            template()
        except ExpressionError:
            exc = sys.exc_info()[1]
            self.assertTrue(body[exc.offset:].startswith('bad ///'))
        else:
            self.fail("Expected exception")

    @error("""<tal:dummy attributes=\"dummy 'dummy'\" />""")
    def test_attributes_on_tal_tag_fails(self, body, exc):
        self.assertTrue(body[exc.offset:].startswith('dummy'))

    @error("""<tal:dummy i18n:attributes=\"foo, bar\" />""")
    def test_i18n_attributes_with_non_identifiers(self, body, exc):
        self.assertTrue(body[exc.offset:].startswith('foo,'))

    @error("""<tal:dummy repeat=\"key,value mydict.items()\">""")
    def test_repeat_syntax_error_message(self, body, exc):
        self.assertTrue(body[exc.offset:].startswith('key,value'))

    @error('''<tal:dummy><p i18n:translate="mymsgid">
            <span i18n:name="repeat"/><span i18n:name="repeat"/>
            </p></tal:dummy>''')
    def test_repeat_i18n_name_error(self, body, exc):
        self.assertTrue(body[exc.offset:].startswith('repeat'), body[exc.offset:])

    @error('''<tal:dummy>
            <span i18n:name="not_in_translation"/>
            </tal:dummy>''')
    def test_i18n_name_not_in_translation_error(self, body, exc):
        self.assertTrue(body[exc.offset:].startswith('not_in_translation'))

    def test_encoded(self):
        filename = '074-encoded-template.pt'
        with open(os.path.join(self.root, 'inputs', filename), 'rb') as f:
            body = f.read()

        self.from_string(body)

    def test_utf8_encoded(self):
        filename = '073-utf8-encoded.pt'
        with open(os.path.join(self.root, 'inputs', filename), 'rb') as f:
            body = f.read()

        self.from_string(body)

    def test_unicode_decode_error(self):
        template = self.from_file(
            os.path.join(self.root, 'inputs', 'greeting.pt')
            )

        string = native = "the artist formerly known as ƤŗíƞĆě"
        try:
            string = string.decode('utf-8')
        except AttributeError:
            pass

        class name:
            @staticmethod
            def __html__():
                # This raises a decoding exception
                string.encode('utf-8').decode('ascii')

                self.fail("Expected exception raised.")

        try:
            template(name=name)
        except UnicodeDecodeError:
            exc = sys.exc_info()[1]
            formatted = str(exc)

            # There's a marker under the expression that has the
            # unicode decode error
            self.assertTrue('^^^^^' in formatted)
            self.assertTrue(native in formatted)
        else:
            self.fail("expected error")

    def test_custom_encoding_for_str_or_bytes_in_content(self):
        string = '<div>Тест${text}</div>'
        try:
            string = string.decode('utf-8')
        except AttributeError:
            pass

        template = self.from_string(string, encoding="windows-1251")

        text = 'Тест'

        try:
            text = text.decode('utf-8')
        except AttributeError:
            pass

        rendered = template(text=text.encode('windows-1251'))

        self.assertEqual(
            rendered,
            string.replace('${text}', text)
            )

    def test_custom_encoding_for_str_or_bytes_in_attributes(self):
        string = '<img tal="Тест${text}" />'
        try:
            string = string.decode('utf-8')
        except AttributeError:
            pass

        template = self.from_string(string, encoding="windows-1251")

        text = 'Тест'

        try:
            text = text.decode('utf-8')
        except AttributeError:
            pass

        rendered = template(text=text.encode('windows-1251'))

        self.assertEqual(
            rendered,
            string.replace('${text}', text)
            )

    def test_null_translate_function(self):
        template = self.from_string('${test}', translate=None)
        rendered = template(test=object())
        self.assertTrue('object' in rendered)

    def test_object_substitution_coerce_to_str(self):
        template = self.from_string('${test}', translate=None)

        class dummy(object):
            def __repr__(inst):
                self.fail("call not expected")

            def __str__(inst):
                return '<dummy>'

        rendered = template(test=dummy())
        self.assertEqual(rendered, '&lt;dummy&gt;')

    def test_repr(self):
        template = self.from_file(
            os.path.join(self.root, 'inputs', 'hello_world.pt')
            )
        self.assertTrue(template.filename in repr(template))

    def test_underscore_variable(self):
        template = self.from_string(
            "<div tal:define=\"_dummy 'foo'\">${_dummy}</div>"
            )
        self.assertTrue(template(), "<div>foo</div>")

    def test_trim_attribute_space(self):
        document = '''<div
                  class="document"
                  id="test"
                  tal:attributes="class string:${default} test"
            />'''

        result1 = self.from_string(
            document)()

        result2 = self.from_string(
            document, trim_attribute_space=True)()

        self.assertEqual(result1.count(" "), 49)
        self.assertEqual(result2.count(" "), 4)
        self.assertTrue(" />" in result1)
        self.assertTrue(" />" in result2)

    def test_exception(self):
        from traceback import format_exception_only

        template = self.from_string(
            "<div tal:define=\"dummy foo\">${dummy}</div>"
            )
        try:
            template()
        except:
            exc = sys.exc_info()[1]
            formatted = str(exc)
            self.assertFalse('NameError:' in formatted)
            self.assertTrue('foo' in formatted)
            self.assertTrue('(line 1: col 23)' in formatted)

            formatted_exc = "\n".join(format_exception_only(type(exc), exc))
            self.assertTrue('NameError: foo' in formatted_exc)
        else:
            self.fail("expected error")

    def test_create_formatted_exception(self):
        from chameleon.utils import create_formatted_exception

        exc = create_formatted_exception(NameError('foo'), NameError, str)
        self.assertEqual(exc.args, ('foo', ))

        class MyNameError(NameError):
            def __init__(self, boo):
                NameError.__init__(self, boo)
                self.bar = boo

        exc = create_formatted_exception(MyNameError('foo'), MyNameError, str)
        self.assertEqual(exc.args, ('foo', ))
        self.assertEqual(exc.bar, 'foo')

    def test_create_formatted_exception_no_subclass(self):
        from chameleon.utils import create_formatted_exception

        class DifficultMetaClass(type):
            def __init__(self, class_name, bases, namespace):
                if not bases == (BaseException, ):
                    raise TypeError(bases)

        Difficult = DifficultMetaClass('Difficult', (BaseException, ), {'args': ()})

        exc = create_formatted_exception(Difficult(), Difficult, str)
        self.assertEqual(exc.args, ())

    def test_error_handler_makes_safe_copy(self):
        calls = []

        class TestException(Exception):
            def __init__(self, *args, **kwargs):
                calls.append((args, kwargs))

        def _render(stream, econtext, rcontext):
            exc = TestException('foo', bar='baz')
            rcontext['__error__'] = ('expression', 1, 42, 'test.pt', exc),
            raise exc

        template = self.from_string("")
        template._render = _render
        try:
            template()
        except TestException:
            self.assertEqual(calls, [(('foo', ), {'bar': 'baz'})])
            exc = sys.exc_info()[1]
            formatted = str(exc)
            self.assertTrue('TestException' in formatted)
            self.assertTrue('"expression"' in formatted)
            self.assertTrue('(line 1: col 42)' in formatted)
        else:
            self.fail("unexpected error")

    def test_double_underscore_variable(self):
        from chameleon.exc import TranslationError
        self.assertRaises(
            TranslationError, self.from_string,
            "<div tal:define=\"__dummy 'foo'\">${__dummy}</div>",
            )

    def test_compiler_internals_are_disallowed(self):
        from chameleon.compiler import COMPILER_INTERNALS_OR_DISALLOWED
        from chameleon.exc import TranslationError

        for name in COMPILER_INTERNALS_OR_DISALLOWED:
            body = "<d tal:define=\"%s 'foo'\">${%s}</d>" % (name, name)
            self.assertRaises(TranslationError, self.from_string, body)

    def test_simple_translate_mapping(self):
        template = self.from_string(
            '<div i18n:translate="">'
            '<span i18n:name="name">foo</span>'
            '</div>')

        self.assertEqual(template(), '<div><span>foo</span></div>')

    def test_translate_is_not_an_internal(self):
        macro = self.from_string('<span i18n:translate="">bar</span>')
        template = self.from_string(
            '''
            <tal:defs define="translate string:">
              <span i18n:translate="">foo</span>
              <metal:macro use-macro="macro" />
            </tal:defs>
            ''')

        result = template(macro=macro)
        self.assertTrue('foo' in result)
        self.assertTrue('foo' in result)

    def test_literal_false(self):
        template = self.from_string(
            '<input type="input" tal:attributes="checked False" />'
            '<input type="input" tal:attributes="checked True" />'
            '<input type="input" tal:attributes="checked None" />'
            '<input type="input" tal:attributes="checked default" />',
            literal_false=True,
            )

        self.assertEqual(
            template(),
            '<input type="input" checked="False" />'
            '<input type="input" checked="True" />'
            '<input type="input" />'
            '<input type="input" />',
            template.source
            )

    def test_boolean_attributes(self):
        template = self.from_string(
            '<input type="input" tal:attributes="checked False" />'
            '<input type="input" tal:attributes="checked True" />'
            '<input type="input" tal:attributes="checked None" />'
            '<input type="input" tal:attributes="checked \'\'" />'
            '<input type="input" tal:attributes="checked default" />'
            '<input type="input" checked="checked" tal:attributes="checked default" />',
            boolean_attributes=set(['checked'])
            )

        self.assertEqual(
            template(),
            '<input type="input" />'
            '<input type="input" checked="checked" />'
            '<input type="input" />'
            '<input type="input" />'
            '<input type="input" />'
            '<input type="input" checked="checked" />',
            template.source
            )

    def test_default_debug_flag(self):
        from chameleon.config import DEBUG_MODE
        template = self.from_file(
            os.path.join(self.root, 'inputs', 'hello_world.pt'),
            )
        self.assertEqual(template.debug, DEBUG_MODE)
        self.assertTrue('debug' not in template.__dict__)

    def test_debug_flag_on_string(self):
        from chameleon.loader import ModuleLoader

        with open(os.path.join(self.root, 'inputs', 'hello_world.pt')) as f:
            source = f.read()

        template = self.from_string(source, debug=True)

        self.assertTrue(template.debug)
        self.assertTrue(isinstance(template.loader, ModuleLoader))

    def test_debug_flag_on_file(self):
        from chameleon.loader import ModuleLoader
        template = self.from_file(
            os.path.join(self.root, 'inputs', 'hello_world.pt'),
            debug=True,
            )
        self.assertTrue(template.debug)
        self.assertTrue(isinstance(template.loader, ModuleLoader))

    def test_tag_mismatch(self):
        from chameleon.exc import ParseError

        try:
            self.from_string("""
            <div metal:use-macro="layout">
            <div metal:fill-slot="name"></dav>
            </div>
            """)
        except ParseError:
            exc = sys.exc_info()[1]
            self.assertTrue("</dav>" in str(exc))
        else:
            self.fail("Expected error.")


class ZopeTemplatesTestSuite(RenderTestCase):
    def setUp(self):
        self.temp_path = temp_path = tempfile.mkdtemp()

        @self.addCleanup
        def cleanup(path=temp_path):
            shutil.rmtree(path)

    def test_pt_files(self):
        from ..zpt.template import PageTemplateFile

        class Literal(object):
            def __init__(self, s):
                self.s = s

            def __html__(self):
                return self.s

            def __str__(self):
                raise RuntimeError(
                    "%r is a literal." % self.s)

        from chameleon.loader import TemplateLoader
        loader = TemplateLoader(os.path.join(self.root, "inputs"))

        self.execute(
            ".pt", PageTemplateFile,
            literal=Literal("<div>Hello world!</div>"),
            content="<div>Hello world!</div>",
            message=Message(),
            load=loader.bind(PageTemplateFile),
            )

    def test_txt_files(self):
        from ..zpt.template import PageTextTemplateFile
        self.execute(".txt", PageTextTemplateFile)

    def execute(self, ext, factory, **kwargs):
        def translate(msgid, domain=None, mapping=None, context=None,
                      target_language=None, default=None):
            if default is None:
                default = str(msgid)

            if isinstance(msgid, Message):
                default = "Message"

            if mapping:
                default = re.sub(r'\${([a-z_]+)}', r'%(\1)s', default) % \
                          mapping

            if target_language is None:
                return default

            if domain is None:
                with_domain = ""
            else:
                with_domain = " with domain '%s'" % domain

            if context is None:
                with_context = ""
            else:
                with_context = ", context '%s'" % context

            stripped = default.rstrip('\n ')
            return "%s ('%s' translation into '%s'%s%s)%s" % (
                stripped, msgid, target_language, with_domain, with_context,
                default[len(stripped):]
                )

        for input_path, output_path, language in self.find_files(ext):
            # Make friendly title so we can locate the generated
            # source when debugging
            self.shortDescription = lambda: input_path

            # When input path contaiins the string 'implicit-i18n', we
            # enable "implicit translation".
            implicit_i18n = 'implicit-i18n' in input_path
            implicit_i18n_attrs = ("alt", "title") if implicit_i18n else ()

            template = factory(
                input_path,
                keep_source=True,
                strict=False,
                implicit_i18n_translate=implicit_i18n,
                implicit_i18n_attributes=implicit_i18n_attrs,
                )

            params = kwargs.copy()
            params.update({
                'translate': translate,
                'target_language': language,
                })

            template.cook_check()

            try:
                got = template.render(**params)
            except:
                import traceback
                e = traceback.format_exc()
                self.fail("%s\n\n    Example source:\n\n%s" % (e, "\n".join(
                    ["%#03.d%s" % (lineno + 1, line and " " + line or "")
                     for (lineno, line) in
                     enumerate(template.source.split(
                         '\n'))])))

            if isinstance(got, byte_string):
                got = got.decode('utf-8')

            from doctest import OutputChecker
            checker = OutputChecker()

            if not os.path.exists(output_path):
                output = template.body
            else:
                with open(output_path, 'rb') as f:
                    output = f.read()

            from chameleon.utils import read_xml_encoding
            from chameleon.utils import detect_encoding

            if template.content_type == 'text/xml':
                encoding = read_xml_encoding(output) or \
                           template.default_encoding
            else:
                content_type, encoding = detect_encoding(
                    output, template.default_encoding)

            want = output.decode(encoding)

            if checker.check_output(want, got, 0) is False:
                from doctest import Example
                example = Example(input_path, want)
                diff = checker.output_difference(
                    example, got, 0)
                self.fail("(%s) - \n%s\n\nCode:\n%s" % (
                    input_path, diff.rstrip('\n'),
                    template.source.encode('utf-8')))



# -*- coding: utf-8 -*-
pass
import sys as _sys
pass
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')

    # <Expression u"'Hello world!'" (1:2)> -> _content_139955154988272
    try:
        _content_139955154988272 = 'Hello world!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello world!'", 1, 2, '<string>', _sys.exc_info()[1], ))
        raise

    if (_content_139955154988272 is not None):
        _tt = type(_content_139955154988272)
        if ((_tt is int) or (_tt is float) or (_tt is long)):
            _content_139955154988272 = str(_content_139955154988272)
        else:
            if (_tt is str):
                _content_139955154988272 = decode(_content_139955154988272)
            else:
                if (_tt is not unicode):
                    try:
                        _content_139955154988272 = _content_139955154988272.__html__
                    except AttributeError:
                        _content_139955154988272 = convert(_content_139955154988272)
                    else:
                        _content_139955154988272 = _content_139955154988272()
    _content_139955154988272 = ('%s%s' % ((_content_139955154988272 if (_content_139955154988272 is not None) else ''), (u'\n' if (u'\n' is not None) else ''), ))
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-
pass
from chameleon.utils import DebuggingOutputStream as _DebuggingOutputStream
pass
_static_37141584 = {u'class': u'test', }
_static_38400080 = {}
_static_36787472 = {}
_static_36787536 = {}
_static_37142224 = {u'class': u'test', }
_static_38401744 = {}
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_35899728 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x249f050> name=None at 249fdd0> -> _value
    _value = _static_38400080
    econtext['attrs'] = _value

    # <html ... (1:0)
    # --------------------------------------------------------
    append(u'<html>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _previous_i18n_domain_37140368 = _i18n_domain
    _i18n_domain = u'old'
    _backup_attrs_35899296 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x249f6d0> name=None at 249f850> -> _value
    _value = _static_38401744
    econtext['attrs'] = _value

    # <body ... (2:2)
    # --------------------------------------------------------
    append(u'<body>')
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _previous_i18n_domain_36787664 = _i18n_domain
    _i18n_domain = u'new'
    _backup_attrs_35899656 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2315550> name=None at 2315d90> -> _value
    _value = _static_36787536
    econtext['attrs'] = _value

    # <div ... (3:4)
    # --------------------------------------------------------
    append(u'<div>')
    _stream_36789456 = _DebuggingOutputStream()
    _append_36789456 = _stream_36789456.append
    _content_139955154988272 = u'\n      Hello world!\n    '
    if (_content_139955154988272 is not None):
        _append_36789456(_content_139955154988272)
    _msgid_36789456 = re_whitespace(''.join(_stream_36789456)).strip()
    append(translate(_msgid_36789456, mapping=None, default=_msgid_36789456, domain=_i18n_domain))
    append(u'</div>')
    if (_backup_attrs_35899656 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_35899656
    _i18n_domain = _previous_i18n_domain_36787664
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38614584 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x2315510> name=None at 23153d0> -> _value
    _value = _static_36787472
    econtext['attrs'] = _value

    # <div ... (6:4)
    # --------------------------------------------------------
    append(u'<div>')
    _stream_36787280 = _DebuggingOutputStream()
    _append_36787280 = _stream_36787280.append
    _content_139955154988272 = u'\n      Hello world!\n    '
    if (_content_139955154988272 is not None):
        _append_36787280(_content_139955154988272)
    _msgid_36787280 = re_whitespace(''.join(_stream_36787280)).strip()
    append(translate(_msgid_36787280, mapping=None, default=_msgid_36787280, domain=_i18n_domain))
    append(u'</div>')
    if (_backup_attrs_38614584 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38614584
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _previous_i18n_domain_37139408 = _i18n_domain
    _i18n_domain = u'new'
    _backup_attrs_36648648 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x236bc50> name=None at 24ae750> -> _value
    _value = _static_37141584
    econtext['attrs'] = _value

    # <div ... (9:4)
    # --------------------------------------------------------
    append(u'<div')
    _backup_default_36650880 = get('default', _marker)
    _value = u'test'
    econtext['default'] = _value

    # <Translate msgid=None node=<_ast.Str object at 0x236bf50> at 236b210> -> _attr_class
    _attr_class = u'test'
    _attr_class = translate(_attr_class, default=_attr_class, domain=_i18n_domain)
    if (_attr_class is None):
        pass
    else:
        if (_attr_class is False):
            _attr_class = None
        else:
            _tt = type(_attr_class)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_class = unicode(_attr_class)
            else:
                try:
                    if (_tt is str):
                        _attr_class = decode(_attr_class)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_class = _attr_class.__html__
                            except:
                                _attr_class = convert(_attr_class)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_class = _attr_class()
                else:
                    if ((_attr_class is not None) and (re_needs_escape(_attr_class) is not None)):
                        if ('&' in _attr_class):
                            if (';' in _attr_class):
                                _attr_class = re_amp.sub('&amp;', _attr_class)
                            else:
                                _attr_class = _attr_class.replace('&', '&amp;')
                        if ('<' in _attr_class):
                            _attr_class = _attr_class.replace('<', '&lt;')
                        if ('>' in _attr_class):
                            _attr_class = _attr_class.replace('>', '&gt;')
                        if (u'"' in _attr_class):
                            _attr_class = _attr_class.replace(u'"', '&#34;')
    if (_attr_class is not None):
        append((u' class="%s"' % _attr_class))
    if (_backup_default_36650880 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_36650880
    append(u'>')
    _content_139955154988272 = u'\n      Hello world!\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_36648648 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_36648648
    _i18n_domain = _previous_i18n_domain_37139408
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _previous_i18n_domain_37139344 = _i18n_domain
    _i18n_domain = u'new'
    _backup_attrs_39044808 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x236bed0> name=None at 236b4d0> -> _value
    _value = _static_37142224
    econtext['attrs'] = _value

    # <div ... (12:4)
    # --------------------------------------------------------
    append(u'<div')
    _backup_default_39044664 = get('default', _marker)
    _value = u'test'
    econtext['default'] = _value

    # <Translate msgid=u'test_msgid' node=<_ast.Str object at 0x236b650> at 236b7d0> -> _attr_class
    _attr_class = u'test'
    _attr_class = translate(u'test_msgid', default=_attr_class, domain=_i18n_domain)
    if (_attr_class is None):
        pass
    else:
        if (_attr_class is False):
            _attr_class = None
        else:
            _tt = type(_attr_class)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_class = unicode(_attr_class)
            else:
                try:
                    if (_tt is str):
                        _attr_class = decode(_attr_class)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_class = _attr_class.__html__
                            except:
                                _attr_class = convert(_attr_class)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_class = _attr_class()
                else:
                    if ((_attr_class is not None) and (re_needs_escape(_attr_class) is not None)):
                        if ('&' in _attr_class):
                            if (';' in _attr_class):
                                _attr_class = re_amp.sub('&amp;', _attr_class)
                            else:
                                _attr_class = _attr_class.replace('&', '&amp;')
                        if ('<' in _attr_class):
                            _attr_class = _attr_class.replace('<', '&lt;')
                        if ('>' in _attr_class):
                            _attr_class = _attr_class.replace('>', '&gt;')
                        if (u'"' in _attr_class):
                            _attr_class = _attr_class.replace(u'"', '&#34;')
    if (_attr_class is not None):
        append((u' class="%s"' % _attr_class))
    if (_backup_default_39044664 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_39044664
    append(u'>')
    _content_139955154988272 = u'\n      Hello world!\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</div>')
    if (_backup_attrs_39044808 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_39044808
    _i18n_domain = _previous_i18n_domain_37139344
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</body>')
    if (_backup_attrs_35899296 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_35899296
    _i18n_domain = _previous_i18n_domain_37140368
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</html>')
    if (_backup_attrs_35899728 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_35899728
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass


# -*- coding: utf-8 -*-
pass
from chameleon.utils import Placeholder as _Placeholder
import sys as _sys
pass
_static_38461520 = {u'alt': u"'Hello world!'", }
_static_38577424 = {}
_static_38427984 = {}
_static_38578896 = {u'alt': u"${'Hello world!'}", }
_static_38401104 = {u'alt': u"'Hello world!'", }
_static_38462480 = {u'alt': u"${'Hello world!'}", }
_static_38428112 = {}
_marker_default = _Placeholder()
import re
import functools
_marker = object()
g_re_amp = re.compile('&(?!([A-Za-z]+|#[0-9]+);)')
g_re_needs_escape = re.compile('[&<>\\"\\\']').search
re_whitespace = functools.partial(re.compile('\\s+').sub, ' ')

def render(stream, econtext, rcontext):
    append = stream.append
    getitem = econtext.__getitem__
    get = econtext.get
    _i18n_domain = None
    re_amp = g_re_amp
    re_needs_escape = g_re_needs_escape
    decode = getitem('decode')
    convert = getitem('convert')
    translate = getitem('translate')
    _backup_attrs_37150016 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24a5dd0> name=None at 24a59d0> -> _value
    _value = _static_38428112
    econtext['attrs'] = _value

    # <html ... (1:0)
    # --------------------------------------------------------
    append(u'<html>')
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_36785184 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24a5d50> name=None at 24a5a50> -> _value
    _value = _static_38427984
    econtext['attrs'] = _value

    # <body ... (2:2)
    # --------------------------------------------------------
    append(u'<body>')
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_38523176 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24ca510> name=None at 24ca9d0> -> _value
    _value = _static_38577424
    econtext['attrs'] = _value

    # <div ... (3:4)
    # --------------------------------------------------------
    append(u'<div>')
    _backup_default_36784176 = get('default', _marker)

    # <Marker name='default' at 24cae10> -> _value
    _value = _marker_default
    econtext['default'] = _value

    # <Expression u'string:Hello world!' (3:40)> -> _cache_38578576
    try:
        _cache_38578576 = u'Hello world!'
    except:
        rcontext.setdefault('__error__', []).append((u'string:Hello world!', 3, 40, '<string>', _sys.exc_info()[1], ))
        raise


    # <Identity expression=<Expression u'string:Hello world!' (3:40)> value=<Marker name='default' at 24ca890> at 24ca110> -> _condition
    _expression = _cache_38578576

    # <Marker name='default' at 24ca890> -> _value
    _value = _marker_default
    _condition = (_expression is _value)
    if _condition:
        _content_139955154988272 = u'\n      Hello world!\n    '
        if (_content_139955154988272 is not None):
            append(_content_139955154988272)
    else:
        _content = _cache_38578576
        _content = translate(_content, default=None, domain=_i18n_domain)
        if (_content is not None):
            append(_content)
    if (_backup_default_36784176 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_36784176
    append(u'</div>')
    if (_backup_attrs_38523176 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_38523176
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_36793448 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24caad0> name=None at 24cae90> -> _value
    _value = _static_38578896
    econtext['attrs'] = _value

    # <img ... (6:4)
    # --------------------------------------------------------
    append(u'<img')
    _backup_default_36792152 = get('default', _marker)
    _value = u"${'Hello world!'}"
    econtext['default'] = _value

    # <Translate msgid=None node=<Interpolation value=u"${'Hello world!'}" escape=True at 24ae090> at 24ae190> -> _attr_alt

    # <Interpolation value=u"${'Hello world!'}" escape=True at 24ae090> -> _attr_alt

    # <Expression u"'Hello world!'" (6:16)> -> _attr_alt
    try:
        _attr_alt = 'Hello world!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello world!'", 6, 16, '<string>', _sys.exc_info()[1], ))
        raise

    _attr_alt = _attr_alt
    _attr_alt = translate(_attr_alt, default=_attr_alt, domain=_i18n_domain)
    if (_attr_alt is None):
        pass
    else:
        if (_attr_alt is False):
            _attr_alt = None
        else:
            _tt = type(_attr_alt)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_alt = unicode(_attr_alt)
            else:
                try:
                    if (_tt is str):
                        _attr_alt = decode(_attr_alt)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_alt = _attr_alt.__html__
                            except:
                                _attr_alt = convert(_attr_alt)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_alt = _attr_alt()
                else:
                    if ((_attr_alt is not None) and (re_needs_escape(_attr_alt) is not None)):
                        if ('&' in _attr_alt):
                            if (';' in _attr_alt):
                                _attr_alt = re_amp.sub('&amp;', _attr_alt)
                            else:
                                _attr_alt = _attr_alt.replace('&', '&amp;')
                        if ('<' in _attr_alt):
                            _attr_alt = _attr_alt.replace('<', '&lt;')
                        if ('>' in _attr_alt):
                            _attr_alt = _attr_alt.replace('>', '&gt;')
                        if (u'"' in _attr_alt):
                            _attr_alt = _attr_alt.replace(u'"', '&#34;')
    if (_attr_alt is not None):
        append((u' alt="%s"' % _attr_alt))
    if (_backup_default_36792152 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_36792152
    append(u' />')
    if (_backup_attrs_36793448 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_36793448
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_37151448 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24ae410> name=None at 24ae310> -> _value
    _value = _static_38462480
    econtext['attrs'] = _value

    # <img ... (7:4)
    # --------------------------------------------------------
    append(u'<img')
    _backup_default_37154184 = get('default', _marker)
    _value = u"${'Hello world!'}"
    econtext['default'] = _value

    # <Translate msgid=u'hello_world' node=<Interpolation value=u"${'Hello world!'}" escape=True at 24ae9d0> at 24aecd0> -> _attr_alt

    # <Interpolation value=u"${'Hello world!'}" escape=True at 24ae9d0> -> _attr_alt

    # <Expression u"'Hello world!'" (7:16)> -> _attr_alt
    try:
        _attr_alt = 'Hello world!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello world!'", 7, 16, '<string>', _sys.exc_info()[1], ))
        raise

    _attr_alt = _attr_alt
    _attr_alt = translate(u'hello_world', default=_attr_alt, domain=_i18n_domain)
    if (_attr_alt is None):
        pass
    else:
        if (_attr_alt is False):
            _attr_alt = None
        else:
            _tt = type(_attr_alt)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_alt = unicode(_attr_alt)
            else:
                try:
                    if (_tt is str):
                        _attr_alt = decode(_attr_alt)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_alt = _attr_alt.__html__
                            except:
                                _attr_alt = convert(_attr_alt)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_alt = _attr_alt()
                else:
                    if ((_attr_alt is not None) and (re_needs_escape(_attr_alt) is not None)):
                        if ('&' in _attr_alt):
                            if (';' in _attr_alt):
                                _attr_alt = re_amp.sub('&amp;', _attr_alt)
                            else:
                                _attr_alt = _attr_alt.replace('&', '&amp;')
                        if ('<' in _attr_alt):
                            _attr_alt = _attr_alt.replace('<', '&lt;')
                        if ('>' in _attr_alt):
                            _attr_alt = _attr_alt.replace('>', '&gt;')
                        if (u'"' in _attr_alt):
                            _attr_alt = _attr_alt.replace(u'"', '&#34;')
    if (_attr_alt is not None):
        append((u' alt="%s"' % _attr_alt))
    if (_backup_default_37154184 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_37154184
    append(u' />')
    if (_backup_attrs_37151448 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_37151448
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_37152528 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x24ae050> name=None at 24ae210> -> _value
    _value = _static_38461520
    econtext['attrs'] = _value

    # <img ... (8:4)
    # --------------------------------------------------------
    append(u'<img')
    _backup_default_37151520 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Translate msgid=None node=<Expression u"'Hello world!'" (8:29)> at 24ae750> -> _attr_alt

    # <Expression u"'Hello world!'" (8:29)> -> _attr_alt
    try:
        _attr_alt = 'Hello world!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello world!'", 8, 29, '<string>', _sys.exc_info()[1], ))
        raise

    _attr_alt = translate(_attr_alt, default=_attr_alt, domain=_i18n_domain)
    if (_attr_alt is None):
        pass
    else:
        if (_attr_alt is False):
            _attr_alt = None
        else:
            _tt = type(_attr_alt)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_alt = unicode(_attr_alt)
            else:
                try:
                    if (_tt is str):
                        _attr_alt = decode(_attr_alt)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_alt = _attr_alt.__html__
                            except:
                                _attr_alt = convert(_attr_alt)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_alt = _attr_alt()
                else:
                    if ((_attr_alt is not None) and (re_needs_escape(_attr_alt) is not None)):
                        if ('&' in _attr_alt):
                            if (';' in _attr_alt):
                                _attr_alt = re_amp.sub('&amp;', _attr_alt)
                            else:
                                _attr_alt = _attr_alt.replace('&', '&amp;')
                        if ('<' in _attr_alt):
                            _attr_alt = _attr_alt.replace('<', '&lt;')
                        if ('>' in _attr_alt):
                            _attr_alt = _attr_alt.replace('>', '&gt;')
                        if ('"' in _attr_alt):
                            _attr_alt = _attr_alt.replace('"', '&#34;')
    if (_attr_alt is not None):
        append((u' alt="%s"' % _attr_alt))
    if (_backup_default_37151520 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_37151520
    append(u' />')
    if (_backup_attrs_37152528 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_37152528
    _content_139955154988272 = u'\n    '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    _backup_attrs_37151088 = get('attrs', _marker)

    # <Static value=<_ast.Dict object at 0x249f450> name=None at 249fcd0> -> _value
    _value = _static_38401104
    econtext['attrs'] = _value

    # <img ... (9:4)
    # --------------------------------------------------------
    append(u'<img')
    _backup_default_37154472 = get('default', _marker)
    _value = None
    econtext['default'] = _value

    # <Translate msgid=u'hello_world' node=<Expression u"'Hello world!'" (9:29)> at 249fc50> -> _attr_alt

    # <Expression u"'Hello world!'" (9:29)> -> _attr_alt
    try:
        _attr_alt = 'Hello world!'
    except:
        rcontext.setdefault('__error__', []).append((u"'Hello world!'", 9, 29, '<string>', _sys.exc_info()[1], ))
        raise

    _attr_alt = translate(u'hello_world', default=_attr_alt, domain=_i18n_domain)
    if (_attr_alt is None):
        pass
    else:
        if (_attr_alt is False):
            _attr_alt = None
        else:
            _tt = type(_attr_alt)
            if ((_tt is int) or (_tt is float) or (_tt is long)):
                _attr_alt = unicode(_attr_alt)
            else:
                try:
                    if (_tt is str):
                        _attr_alt = decode(_attr_alt)
                    else:
                        if (_tt is not unicode):
                            try:
                                _attr_alt = _attr_alt.__html__
                            except:
                                _attr_alt = convert(_attr_alt)
                            else:
                                raise RuntimeError
                except RuntimeError:
                    _attr_alt = _attr_alt()
                else:
                    if ((_attr_alt is not None) and (re_needs_escape(_attr_alt) is not None)):
                        if ('&' in _attr_alt):
                            if (';' in _attr_alt):
                                _attr_alt = re_amp.sub('&amp;', _attr_alt)
                            else:
                                _attr_alt = _attr_alt.replace('&', '&amp;')
                        if ('<' in _attr_alt):
                            _attr_alt = _attr_alt.replace('<', '&lt;')
                        if ('>' in _attr_alt):
                            _attr_alt = _attr_alt.replace('>', '&gt;')
                        if ('"' in _attr_alt):
                            _attr_alt = _attr_alt.replace('"', '&#34;')
    if (_attr_alt is not None):
        append((u' alt="%s"' % _attr_alt))
    if (_backup_default_37154472 is _marker):
        del econtext['default']
    else:
        econtext['default'] = _backup_default_37154472
    append(u' />')
    if (_backup_attrs_37151088 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_37151088
    _content_139955154988272 = u'\n  '
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</body>')
    if (_backup_attrs_36785184 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_36785184
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
    append(u'</html>')
    if (_backup_attrs_37150016 is _marker):
        del econtext['attrs']
    else:
        econtext['attrs'] = _backup_attrs_37150016
    _content_139955154988272 = u'\n'
    if (_content_139955154988272 is not None):
        append(_content_139955154988272)
pass